--[[
Copyright (C) GtX (Andy), 2019

Author: GtX | Andy
Date: 12.12.2019
Revision: FS22-01

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Important:
Not to be added to any mods / maps or modified from its current release form.
No modifications may be made to this script, including conversion to other game versions without written permission from GtX | Andy

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
Ohne schriftliche Genehmigung von GtX | Andy dürfen keine Änderungen an diesem Skript vorgenommen werden, einschließlich der Konvertierung in andere Spielversionen
]]


FillLevelLimiterDialog = {}

local FillLevelLimiterDialog_mt = Class(FillLevelLimiterDialog, YesNoDialog)

FillLevelLimiterDialog.NOT_APPLICABLE = "N/A"
FillLevelLimiterDialog.UPDATE_COUNTER_INTERVAL = 2000

FillLevelLimiterDialog.MOD_NAME = g_currentModName

FillLevelLimiterDialog.CONTROLS = {
    "fillLevelTextElement",
    "fillUnitIndexElement",
    "dialogTextElement",
    "settingStateElement",
    "yesButton",
    "noButton",
    "resetButton"
}

function FillLevelLimiterDialog.new()
    local self = YesNoDialog.new(nil, FillLevelLimiterDialog_mt)

    self:registerControls(FillLevelLimiterDialog.CONTROLS)
    self.isButtonDisabled = false

    self.showHelpInfo = true
    self.doEnterPress = false

    self.updateCounter = FillLevelLimiterDialog.UPDATE_COUNTER_INTERVAL

    return self
end

function FillLevelLimiterDialog:setData(vehicle, showHelpInfo)
    self.vehicle = vehicle
    self.fillUnitIndex = nil

    self.lastTextValue = {}
    self.fillLevelLimits = {}

    self.fillUnitSelectorToIndex = {}
    self.fillUnitHeader = g_i18n:getText("ui_fll_fillUnitHeader", FillLevelLimiterDialog.MOD_NAME)

    local fillUnitSelectorNames = {}
    local spec = vehicle.spec_fillLevelLimiter

    local resetDisabled = true

    if spec ~= nil then
        for fillUnitIndex, limit in pairs (spec.fillUnitIndexToLimit) do
            if limit < self.vehicle:getFillUnitCapacity(fillUnitIndex) then
                resetDisabled = false
            end

            self.lastTextValue[fillUnitIndex] = tostring(limit)
            self.fillLevelLimits[fillUnitIndex] = limit

            table.insert(self.fillUnitSelectorToIndex, fillUnitIndex)

            local name = spec.fillUnitTitle[fillUnitIndex] or FillLevelLimiterDialog.NOT_APPLICABLE

            table.insert(fillUnitSelectorNames, name)
        end
    end

    self.fillUnitIndexElement:setTexts(fillUnitSelectorNames)
    self.fillUnitIndexElement:setState(1, true)

    self.showHelpInfo = Utils.getNoNil(showHelpInfo, true)
    self.settingStateElement:setIsChecked(self.showHelpInfo)

    self:setYesButtonDisabled(spec == nil)
    self:setResetButtonDisabled(resetDisabled)
end

function FillLevelLimiterDialog:onEnterPressed()
    if self.fillLevelTextElement.text ~= "" then
        local newFillLevelValue = tonumber(self.fillLevelTextElement.text)

        if newFillLevelValue ~= nil then
            local fillLevel = self.vehicle:getFillUnitFillLevel(self.fillUnitIndex) or 1
            local capacity = self.vehicle:getFillUnitCapacity(self.fillUnitIndex)

            self.fillLevelLimits[self.fillUnitIndex] = MathUtil.clamp(newFillLevelValue, math.max(math.floor(fillLevel + 0.5), 1), capacity)
        end
    end

    self.lastTextValue[self.fillUnitIndex] = string.format("%d", self.fillLevelLimits[self.fillUnitIndex])
    self.fillLevelTextElement:setText(self.lastTextValue[self.fillUnitIndex])
    self.doEnterPress = false
end

function FillLevelLimiterDialog:onEscPressed()
    self.fillLevelTextElement:setText(self:getCurrentTextValue())
    self.doEnterPress = false
end

function FillLevelLimiterDialog:onTextChanged()
    local text = self.fillLevelTextElement.text

    if text ~= self.lastTextValue[self.fillUnitIndex] then
        if text ~= "" then
            if tonumber(text) ~= nil then
                self.lastTextValue[self.fillUnitIndex] = text
            else
                self.fillLevelTextElement:setText(self.lastTextValue[self.fillUnitIndex])
            end
        end

        self.doEnterPress = true
    end
end

function FillLevelLimiterDialog:update(dt)
    FillLevelLimiterDialog:superClass().update(self, dt)

    if self.vehicle ~= nil and self.fillUnitIndex ~= nil then
        self.updateCounter = self.updateCounter - dt

        if self.updateCounter < 0 then
            local fillUnits = self.vehicle:getFillUnits()
            local fillUnit = fillUnits ~= nil and fillUnits[self.fillUnitIndex]

            if fillUnit ~= nil and fillUnit.fillLevel ~= nil then
                if fillUnit.fillLevel ~= self.lastFillLevelForUpdate then
                    self.lastFillLevelForUpdate = fillUnit.fillLevel

                    local fillLevel = math.floor((fillUnit.fillLevel) + 0.5)
                    local capacity = math.floor((fillUnit.capacity or 0) + 0.5)
                    local unitText = fillUnit.unitText or g_i18n:getVolumeUnit(false)

                    self.dialogTextElement:setText(string.format(self.fillUnitHeader, math.max(fillLevel, 1), capacity, unitText))
                end
            end

            self.updateCounter = FillLevelLimiterDialog.UPDATE_COUNTER_INTERVAL
        end
    end
end

function FillLevelLimiterDialog:getCurrentTextValue()
    if self.fillUnitIndex ~= nil and self.fillLevelLimits[self.fillUnitIndex] ~= nil then
        return string.format("%d", self.fillLevelLimits[self.fillUnitIndex])
    end

    return ""
end

function FillLevelLimiterDialog:onClickOk()
    if self.isYesButtonDisabled then
        return
    end

    if self.doEnterPress then
        self:onEnterPressed()
    end

    self:sendCallback(self.fillLevelLimits, self.showHelpInfo)
end

function FillLevelLimiterDialog:onClickBack(forceBack, usedMenuButton)
    self:close()
end

function FillLevelLimiterDialog:onClickCancel()
    for fillUnitIndex, _ in pairs (self.fillLevelLimits) do
        local capacity = self.vehicle:getFillUnitCapacity(fillUnitIndex)

        self.fillLevelLimits[fillUnitIndex] = capacity
        self.lastTextValue[fillUnitIndex] = tostring(capacity)
    end

    self.fillUnitIndexElement:setState(1, true)
    self:onClickOk()
end

function FillLevelLimiterDialog:onClickFillUnitIndex(index)
    local fillUnitIndex = self.fillUnitSelectorToIndex[index]
    local fillUnits = self.vehicle:getFillUnits()

    if fillUnitIndex ~= nil and fillUnits ~= nil then
        local fillUnit = fillUnits[fillUnitIndex]

        if fillUnit ~= nil then
            local fillLevel = math.floor((fillUnit.fillLevel or 1) + 0.5)
            local capacity = math.floor((fillUnit.capacity or 0) + 0.5)
            local unitText = fillUnit.unitText or g_i18n:getVolumeUnit(false)

            self.dialogTextElement:setText(string.format(self.fillUnitHeader, math.max(fillLevel, 1), capacity, unitText))

            self.fillUnitIndex = fillUnitIndex
            self.fillLevelTextElement:setText(self:getCurrentTextValue())
        end
    end
end

function FillLevelLimiterDialog:onClickSettingState(index)
    self.showHelpInfo = index > 1
end

function FillLevelLimiterDialog:sendCallback(fillLevelLimits, showHelpInfo)
    if self.inputDelay < self.time then
        self:close()

        if self.callbackFunc ~= nil then
            if self.target ~= nil then
                self.callbackFunc(self.target, fillLevelLimits, showHelpInfo)
            else
                self.callbackFunc(fillLevelLimits)
            end
        end
    end
end

function FillLevelLimiterDialog:setYesButtonDisabled(disabled)
    self.isYesButtonDisabled = disabled
    self.yesButton:setDisabled(disabled)
end

function FillLevelLimiterDialog:setResetButtonDisabled(disabled)
    self.isResetButtonDisabled = disabled
    self.resetButton:setDisabled(disabled)
end
