--[[
Copyright (C) GtX (Andy), 2019

Author: GtX | Andy
Date: 12.12.2019
Revision: FS22-01

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Important:
Not to be added to any mods / maps or modified from its current release form.
No modifications may be made to this script, including conversion to other game versions without written permission from GtX | Andy

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
Ohne schriftliche Genehmigung von GtX | Andy dürfen keine Änderungen an diesem Skript vorgenommen werden, einschließlich der Konvertierung in andere Spielversionen
]]


local buildId = 1
local versionString = "0.0.0.0"

local modName = g_currentModName or ""
local modDirectory = g_currentModDirectory or ""

local validationFail
local fillLevelLimiter

local invalidVehicleTypes = {
    ["car"] = true,
    ["baler"] = true,
    ["pallet"] = true,
    ["bigBag"] = true,
    ["tractor"] = true,
    ["leveler"] = true,
    ["woodTruck"] = true,
    ["forwarder"] = true,
    ["locomotive"] = true,
    ["teleHandler"] = true,
    ["buyableBale"] = true,
    ["treePlanter"] = true,
    ["strawBlower"] = true,
    ["conveyorBelt"] = true,
    ["balerWrapper"] = true,
    ["baseDrivable"] = true,
    ["loaderVehicle"] = true,
    ["woodHarvester"] = true,
    ["receivingHopper"] = true,
    ["combineDrivable"] = true,
    ["cottonHarvester"] = true,
    ["attachableCombine"] = true,
    ["treeSaplingPallet"] = true,
    ["pickupConveyorBelt"] = true,
    ["selfPropelledMower"] = true,
    ["tractorCrabSteering"] = true,
    ["tractorReverseDriving"] = true,
    ["combineCutterFruitPreparer"] = true,
    ["attachableCombineCutterFruitPreparer"] = true
}

local invalidClassNames = {
    ["Pallet"] = true,
    ["Baler"] = true,
    ["Mower"] = true,
    ["Leveler"] = true,
    ["Combine"] = true,
    ["StrawBlower"] = true,
    ["TreePlanter"] = true,
    ["BaleWrapper"] = true,
    ["InlineWrapper"] = true,
    ["MultipleItemPurchase"] = true
}

local function getIsValidVehicleType(typeName, specializations)
    if invalidVehicleTypes[typeName] then
        return false
    end

    for _, specialization in pairs(specializations) do
        if invalidClassNames[specialization.className] then
            return false
        end
    end

    return true
end

local function typeManagerValidateTypes(typeManager)
    if typeManager.typeName == "vehicle" and fillLevelLimiter ~= nil then
        local specializationName = string.format("%s.fillLevelLimiter", modName)
        local specializationObject = g_specializationManager:getSpecializationObjectByName(specializationName)

        if specializationObject ~= nil then
            for typeName, typeEntry in pairs (typeManager:getTypes()) do
                if typeEntry.specializationsByName[specializationName] == nil then
                    if getIsValidVehicleType(typeName, typeEntry.specializations) and specializationObject.prerequisitesPresent(typeEntry.specializations) then
                        typeManager:addSpecialization(typeName, specializationName)

                        fillLevelLimiter.numInserted = fillLevelLimiter.numInserted + 1
                    end
                end
            end

            if fillLevelLimiter.numInserted > 0 then
                fillLevelLimiter.dialog = FillLevelLimiterDialog.new()

                g_gui:loadGui(modDirectory .. "gui/FillLevelLimiterDialog.xml", "FillLevelLimiterDialog", fillLevelLimiter.dialog)

                Logging.info("Loaded 'Fill Level Limiter' into %d vehicle types", fillLevelLimiter.numInserted)
            end
        else
            Logging.warning("Failed to load specialization 'fillLevelLimiter', mod '%s' may not perform as intended!", modName)
        end
    end
end

local function init()
    if g_globalMods and g_globalMods.fillLevelLimiter then
        Logging.error("Failed to load mod '%s', 'FillLevelLimiter' was already loaded by '%s'!", modName, g_globalMods.fillLevelLimiter.modName or "N/A")

        return
    end

    local isValidMod = false
    local mod = g_modManager:getModByName(modName)

    versionString = mod.version or versionString

    if mod.modName == "FS22_FillLevelLimiter" or mod.modName == "FS22_FillLevelLimiter_update" then
        isValidMod = mod.author and #mod.author == 3
    end

    if isValidMod then
        fillLevelLimiter = {
            numInserted = 0,
            modName = modName,
            version = version,
            buildId = buildId
        }

        source(modDirectory .. "scripts/specializations/events/FillLevelLimiterEvent.lua")
        source(modDirectory .. "scripts/gui/FillLevelLimiterDialog.lua")

        g_specializationManager:addSpecialization("fillLevelLimiter", "FillLevelLimiter", modDirectory .. "scripts/specializations/FillLevelLimiter.lua", nil)

        TypeManager.validateTypes = Utils.prependedFunction(TypeManager.validateTypes, typeManagerValidateTypes)

        if g_globalMods then
            g_globalMods.fillLevelLimiter = fillLevelLimiter
        end
    else
        validationFail = {
            startUpdateTime = 2000,

            update = function(self, dt)
                self.startUpdateTime = self.startUpdateTime - dt

                if self.startUpdateTime < 0 then
                    local text = string.format(g_i18n:getText("fll_loadError", modName), modName, mod.author or "Unknown")

                    if g_dedicatedServerInfo == nil then
                        if not g_gui:getIsGuiVisible() then
                            g_gui:showYesNoDialog({
                                title = string.format("%s - Version %s", mod.title, versionString),
                                text = text,
                                dialogType = DialogElement.TYPE_LOADING,
                                callback = self.openModHubLink,
                                yesText = g_i18n:getText("button_ok"),
                                noText = g_i18n:getText("button_modHubDownload")
                            })
                        end
                    else
                        print("\n" .. text .. "\n    - https://farming-simulator.com/mods.php?lang=en&country=be&title=fs2022&filter=org&org_id=129652&page=0" .. "\n")
                        self.openModHubLink(true)
                    end
                end
            end,

            openModHubLink = function(ignore)
                if ignore == false then
                    local language = g_languageShort
                    local link = "mods.php?lang=en&country=be&title=fs2022&filter=org&org_id=129652&page=0"
                    if language == "de" or language == "fr" then
                        link = "mods.php?lang=" .. language .. "&country=be&title=fs2022&filter=org&org_id=129652&page=0"
                    end

                    openWebFile(link, "")
                end

                removeModEventListener(validationFail)
                validationFail = nil
            end
        }

        addModEventListener(validationFail)
    end
end

init()
